<?php
require 'db.php';
require 'auth.php';

header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: GET, POST, PUT, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

authenticate();

$method = $_SERVER['REQUEST_METHOD'];

switch ($method) {
    case 'GET':
        handleGetRequest();
        break;
    case 'POST':
        handlePostRequest();
        break;
    case 'PUT':
        handlePutRequest();
        break;
    case 'DELETE':
        handleDeleteRequest();
        break;
    case 'OPTIONS':
        http_response_code(200);
        break;
    default:
        http_response_code(405);
        echo json_encode(["message" => "Method Not Allowed"]);
        break;
}

function handleGetRequest() {
    global $pdo;

    header('Content-Type: application/json');

    if (isset($_GET['id'])) {
        $id = $_GET['id'];
        $stmt = $pdo->prepare("SELECT * FROM eventos WHERE id = ?");
        $stmt->execute([$id]);
        $event = $stmt->fetch(PDO::FETCH_ASSOC);

        if ($event) {
            $stmt = $pdo->prepare("SELECT * FROM fotos WHERE evento_id = ? ORDER BY id ASC");
            $stmt->execute([$id]);
            $event['photos'] = $stmt->fetchAll(PDO::FETCH_ASSOC);
            echo json_encode($event);
        } else {
            http_response_code(404);
            echo json_encode(["message" => "Event not found"]);
        }
    } else {
        $stmt = $pdo->query("SELECT * FROM eventos");
        $events = $stmt->fetchAll(PDO::FETCH_ASSOC);

        foreach ($events as &$event) { 
            $stmt = $pdo->prepare("SELECT * FROM fotos WHERE evento_id = ? ORDER BY id ASC");
            $stmt->execute([$event['id']]);
            $event['photos'] = $stmt->fetchAll(PDO::FETCH_ASSOC);
        }

        echo json_encode($events);
    }
}

function handlePostRequest() {
    global $pdo;

    header('Content-Type: application/json');

    $data = json_decode(file_get_contents('php://input'), true);

    if (!validateEventData($data)) {
        http_response_code(400);
        echo json_encode(["message" => "Invalid input data"]);
        return;
    }

    try {
        $pdo->beginTransaction();

        $stmt = $pdo->prepare("INSERT INTO eventos (name, date, photo, title, description, place, reception) VALUES (?, ?, ?, ?, ?, ?, ?)");
        $stmt->execute([$data['name'], $data['date'], $data['photo'], $data['title'], $data['description'], $data['place'], $data['reception']]);
        $eventId = $pdo->lastInsertId();

        foreach ($data['photos'] as $photo) {
            $stmt = $pdo->prepare("INSERT INTO fotos (evento_id, photo) VALUES (?, ?)");
            $stmt->execute([$eventId, $photo]);
        }

        $pdo->commit();
        echo json_encode(["message" => "Event created", "id" => $eventId]);
    } catch (Exception $e) {
        $pdo->rollBack();
        http_response_code(500);
        echo json_encode(["message" => "Error creating event", "error" => $e->getMessage()]);
    }
}

function handlePutRequest() {
    global $pdo;

    header('Content-Type: application/json');

    $data = json_decode(file_get_contents('php://input'), true);

    // Check if 'id' is set in $_GET
    $id = isset($_GET['id']) ? $_GET['id'] : null;
    if (!$id) {
        http_response_code(400); // Bad Request
        echo json_encode(["message" => "Missing 'id' parameter"]);
        return;
    }

    if (!validateEventData($data)) {
        http_response_code(400);
        echo json_encode(["message" => "Invalid input data"]);
        return;
    }

    try {
        // Use a prepared statement to prevent SQL injection
        $stmt = $pdo->prepare("SELECT * FROM eventos WHERE id = ?");
        $stmt->execute([$id]);
        $event = $stmt->fetch(PDO::FETCH_ASSOC);

        if (!$event) {
            http_response_code(404);
            echo json_encode(["message" => "Event not found"]);
            return;
        }

        $pdo->beginTransaction();

        $sql = "UPDATE eventos SET name = ?, date = ?, photo = ?, title = ?, description = ?, place = ?, reception = ? WHERE id = ?";
        $stmt = $pdo->prepare($sql);
        $stmt->execute([$data['name'], $data['date'], $data['photo'], $data['title'], $data['description'], $data['place'], $data['reception'], $id]);

        $stmt = $pdo->prepare("DELETE FROM fotos WHERE evento_id = ?");
        $stmt->execute([$id]);

        foreach ($data['photos'] as $photo) {
            $stmt = $pdo->prepare("INSERT INTO fotos (evento_id, photo) VALUES (?, ?)");
            $stmt->execute([$id, $photo]);
        }

        $pdo->commit();
        echo json_encode(["message" => "Event updated"]);
    } catch (Exception $e) {
        $pdo->rollBack();
        http_response_code(500);
        echo json_encode(["message" => "Error updating event", "error" => $e->getMessage()]);
    }
}

function handleDeleteRequest() {
    global $pdo;

    header('Content-Type: application/json');

    // Check if 'id' is set in $_GET
    $id = isset($_GET['id']) ? $_GET['id'] : null;
    if (!$id) {
        http_response_code(400); // Bad Request
        echo json_encode(["message" => "Missing 'id' parameter"]);
        return;
    }

    try {
        // Iniciar la transacción
        $pdo->beginTransaction();

        // Eliminar las fotos relacionadas con el evento
        $stmt = $pdo->prepare("DELETE FROM fotos WHERE evento_id = ?");
        $stmt->execute([$id]);

        // Eliminar el evento
        $stmt = $pdo->prepare("DELETE FROM eventos WHERE id = ?");
        $stmt->execute([$id]);

        // Confirmar la transacción
        $pdo->commit();
        echo json_encode(["message" => "Event deleted"]);
    } catch (Exception $e) {
        $pdo->rollBack();
        http_response_code(500);
        echo json_encode(["message" => "Error deleting event", "error" => $e->getMessage()]);
    }
}

function validateEventData($data) {
    return isset($data['name'], $data['date'], $data['photo'], $data['title'], $data['description'], $data['place'], $data['reception'], $data['photos']) &&
           !empty($data['name']) &&
           !empty($data['date']) &&
           !empty($data['photo']) &&
           !empty($data['title']) &&
           !empty($data['description']) &&
           !empty($data['place']) &&
           !empty($data['reception']) &&
           is_array($data['photos']) && 
           count($data['photos']) > 0; 
}